#!/usr/bin/env python
# coding: utf-8

"""
    KATs -- Known Answer Tests
    
"""
import pyPhelix

#---------------------------------------------------------------
# utilities

def loadfmfile(filename):
    f = open(filename, 'rb')
    content = f.read()
    f.close()
    return content

def hex2byt(h):
    if h[-1:] == 'L':
        h = h[:-1]
    if h[:2] == '0x':
        h = h[2:]
    if len(h) % 2:
        h = '0'+h
    bs = ''
    while h:
        bs += chr(eval('0x'+h[:2]))
        h = h[2:]
    return bs

def byt2hex(bs):
    h = ''
    for c in bs:
        h = h + ('0'+hex(ord(c))[2:])[-2:]
    return h

#---------------------------------------------------------------

def encrypt(key, nonce, pText, macSize, aad):
    ph = pyPhelix.Phelix(key, nonce, macSize)
    if aad:
        ph.processAAD(aad)
    cText = ph.encryptBytes(pText)
    mac = ph.getMAC()
    return cText, mac

def decrypt(key, nonce, cText, macSize, aad):
    ph = pyPhelix.Phelix(key, nonce, macSize)
    if aad:
        ph.processAAD(aad)
    pText = ph.decryptBytes(cText)
    mac = ph.getMAC()
    return pText, mac
    
def encryptPacket(key, nonce, pText, macSize, aad):
    return pyPhelix.encryptPacket(key, nonce, pText, macSize, aad)

def decryptPacket(key, nonce, cText, macSize, aad):
    return pyPhelix.decryptPacket(key, nonce, cText, macSize, aad)
    
def testKAT(i, kat):
    (keySize, msgLen, aadLen, macSize, 
     nonceSize, key, nonce, aad, pText, cText, mac) = kat
    key   = hex2byt(key)
    nonce = hex2byt(nonce)
    aad   = hex2byt(aad)
    pText = hex2byt(pText)
    cText = hex2byt(cText)
    mac   = hex2byt(mac)
    good  = 1

    cText2, mac2 = encrypt(key, nonce, pText, macSize, aad)
    if cText2 != cText:
        print '  KAT %3d - failed test1' % i
        print '     expected:', byt2hex(cText)
        print '       actual:', byt2hex(cText2)
        good = 0
    if mac2 != mac:
        print '  KAT %3d - failed test2' % i
        print '     expected:', byt2hex(mac)
        print '       actual:', byt2hex(mac2)
        good = 0
    pText3, mac3 = decrypt(key, nonce, cText, macSize, aad)
    if pText3 != pText:
        print '  KAT %3d - failed test3' % i
        print '     expected:', byt2hex(pText)
        print '       actual:', byt2hex(pText3)
        good = 0
    if mac3 != mac:
        print '  KAT %3d - failed test4' % i
        print '     expected:', byt2hex(mac)
        print '       actual:', byt2hex(mac3)
        good = 0

    cText2, mac2 = encryptPacket(key, nonce, pText, macSize, aad)
    if cText2 != cText:
        print '  KAT %3d - failed test5' % i
        print '     expected:', byt2hex(cText)
        print '       actual:', byt2hex(cText2)
        good = 0
    if mac2 != mac:
        print '  KAT %3d - failed test6' % i
        print '     expected:', byt2hex(mac)
        print '       actual:', byt2hex(mac2)
        good = 0
    pText3, mac3 = decryptPacket(key, nonce, cText, macSize, aad)
    if pText3 != pText:
        print '  KAT %3d - failed test7' % i
        print '     expected:', byt2hex(pText)
        print '       actual:', byt2hex(pText3)
        good = 0
    if mac3 != mac:
        print '  KAT %3d - failed test8' % i
        print '     expected:', byt2hex(mac)
        print '       actual:', byt2hex(mac3)
        good = 0
    return good


def testKATs():
    kats = eval(str(loadfmfile('KAT.txt').split('\n')))
    i = j = 0
    for kat in kats:
        i += 1
        kat = eval(kat)
        j += testKAT(i, kat)
    print '  %d of %d Known Answer Tests successful' % (j, i)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if __name__ == '__main__':
    testKATs()

#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
